@@ -36,6 +36,7 @@
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-mime-handlers.h"
+#include "gnome-vfs-mime-private.h"
 #include <glib/gstrfuncs.h>
 #include <glib/gutils.h>
 #include <gconf/gconf-client.h>
@@ -87,7 +88,7 @@ gnome_vfs_format_file_size_for_display (GnomeVFSFileSize size)
 
 		if (size < (GnomeVFSFileSize) MEGABYTE_FACTOR) {
 			displayed_size = (gdouble) size / KILOBYTE_FACTOR;
-			return g_strdup_printf (_("%.1f K"),
+			return g_strdup_printf (_("%.1f KB"),
 						       displayed_size);
 		} else if (size < (GnomeVFSFileSize) GIGABYTE_FACTOR) {
 			displayed_size = (gdouble) size / MEGABYTE_FACTOR;
@@ -807,6 +808,7 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
 
 	/* ncpfs does not know the amount of available and free space */
 	if (statfs_buffer.f_bavail == 0 && statfs_buffer.f_bfree == 0) {
+#if defined(__linux__)
 		/* statvfs does not contain an f_type field, we try again
 		 * with statfs.
 		 */
@@ -819,7 +821,11 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
 		}
 		
 		/* linux/ncp_fs.h: NCP_SUPER_MAGIC == 0x564c */
-		if (statfs_buffer2.f_type == 0x564c) {
+		if (statfs_buffer2.f_type == 0x564c)
+#elif defined(__sun)
+		if (strcmp(statfs_buffer.f_basetype, "ncpfs") == 0)
+#endif
+		{
 			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		}
 	}
@@ -2046,7 +2052,7 @@ gnome_vfs_url_show_with_env (const char  *url,
 	
 	g_free (scheme);
 
-	type = gnome_vfs_get_mime_type (url);
+	type = _gnome_vfs_get_slow_mime_type (url);
 
 	if (type == NULL) {
 		return GNOME_VFS_ERROR_BAD_PARAMETERS;
